@@ -1,3 +1,5 @@
+/* $Id: glstuff.c,v 1.12 2003/03/22 01:46:37 d3august Exp $
+*/
 /*  xtraceroute - graphically show traceroute information.
  *  Copyright (C) 1996-1998  Bjrn Augustsson 
  *
@@ -16,7 +18,6 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <netinet/in.h>
 #include <stdlib.h>
 #include <gtkgl/gtkglarea.h>
 #include <GL/gl.h>
@@ -26,7 +27,10 @@
 
 extern void sphere(int level);
 
-extern texture *earth_texture;
+extern GdkPixbuf *earth_texture;
+extern GdkPixbuf *night_texture;
+extern GdkPixbuf *created_texture;
+
 //extern struct traceroute_stuff;  // from extprog.c
 extern GLfloat zoom;
 
@@ -34,23 +38,22 @@ static GLUquadricObj *lineobj;
 static GLUquadricObj *earthobj;
 static GLUquadricObj *sitemarker;
 
-const static GLfloat hiaccColor[]    = {0.1, 1.0,  0.1, 1.0};      /* Green for good. */
-const static GLfloat medaccColor[]   = {1.0, 1.0,  0.1, 1.0};      /* Yellow for OK.  */
-const static GLfloat loaccColor[]    = {1.0, 0.53, 0.0, 1.0};      /* Orange for bad  */
-const static GLfloat noaccColor[]    = {1.0, 0.1,  0.1, 1.0};      /* Red for no info */
+static const GLfloat hiaccColor[]  = {0.1, 1.0,  0.1, 1.0}; // Green for good.
+static const GLfloat medaccColor[] = {1.0, 1.0,  0.1, 1.0}; // Yellow for OK.
+static const GLfloat loaccColor[]  = {1.0, 0.53, 0.0, 1.0}; // Orange for bad
+static const GLfloat noaccColor[]  = {1.0, 0.1,  0.1, 1.0}; // Red for no info
 
-const static GLfloat lineColor[]         = {1.0, 1.0, 0.1, 1.0};      /* Yellow */
-const static GLfloat defaultColor[]      = {0.8, 0.8, 0.8, 1.0};
+static const GLfloat lineColor[]         = {1.0, 1.0, 0.1, 1.0}; // Yellow
+static const GLfloat defaultColor[]      = {0.8, 0.8, 0.8, 1.0};
 
-const static GLfloat selectedEmission[]  = {0.6, 0.6, 0.6, 1.0};
-const static GLfloat defaultEmission[]   = {0.0, 0.0, 0.0, 1.0};
+static const GLfloat selectedEmission[]  = {0.6, 0.6, 0.6, 1.0};
+static const GLfloat defaultEmission[]   = {0.0, 0.0, 0.0, 1.0};
 
-const static GLfloat lightZeroPosition[] = { 10.0,  4.0, 10.0, 1.0};
-const static GLfloat lightZeroColor[]    = {  0.8,  1.0,  0.8, 1.0}; /* green-tinted */
-const static GLfloat lightOnePosition[]  = { -1.0, -2.0,  1.0, 0.0};
-const static GLfloat lightOneColor[]     = {  0.9,  0.9,  0.9, 1.0}; /* bright white */
+static const GLfloat lightZeroPosition[] = { 10.0,  4.0, 10.0, 1.0};
+static const GLfloat lightZeroColor[]    = {  0.8,  1.0,  0.8, 1.0}; /* green-tinted */
+static const GLfloat lightOnePosition[]  = { -1.0, -2.0,  1.0, 0.0};
+static const GLfloat lightOneColor[]     = {  0.9,  0.9,  0.9, 1.0}; /* bright white */
 
-static int sphere_lod = 3;
 static int render_mode = NORMAL_MODE;
 
 /**
@@ -62,7 +65,7 @@ static int render_mode = NORMAL_MODE;
 void
 set_sphere_lod(const int lod)
 {
-  sphere_lod = lod;
+  user_settings->LOD = lod;
 }
 
 /**
@@ -101,10 +104,10 @@ between(const GLdouble lat1, const GLdouble lon1,
 /*  'Normalize' to the radius of the earth. (Which is always 1, since this
      is the Unit Earth (TM))  */
 
-   GLdouble f  = 1 / sqrt(x*x + y*y + z*z);
-    x*=f;
-    y*=f;
-    z*=f;
+   GLdouble f = 1.0 / sqrt(x*x + y*y + z*z);
+    x *= f;
+    y *= f;
+    z *= f;
     *rlat = tolat(x,y,z);
     *rlon = tolon(x,y,z);
   }
@@ -169,7 +172,7 @@ plot_line(const GLfloat fromlat, const GLfloat fromlon, const GLfloat tolat, con
        sitemarker we just drew.             */
   
   if(render_mode == SELECT_MODE)
-    glLoadName(-1);
+    glLoadName(NOT_SELECTABLE);
   
   partialtorus((GLfloat)fromlat-180.0, (GLfloat)linesweep, zoom);
   glPopMatrix();
@@ -188,13 +191,13 @@ static void
 plot_sat(const GLfloat fromlat, const GLfloat fromlon, 
 	 const GLfloat destlat, const GLfloat destlon)
 {
-  const static GLfloat satPanelColor[]  = { 0.0, 0.0, 0.2, 1.0}; /* Glossy dark-blue */
-  const static GLfloat satBodyColor[]   = {1.0, 0.53, 0.0, 1.0}; /* Goldish */
-  const static GLfloat mat_specular[]   = { 1.0, 1.0, 1.0, 1.0 };
-  const static GLfloat mat_nospecular[] = { 0.0, 0.0, 0.0, 1.0 };
+  static const GLfloat satPanelColor[]  = { 0.0, 0.0, 0.2, 1.0}; /* Glossy dark-blue */
+  static const GLfloat satBodyColor[]   = {1.0, 0.53, 0.0, 1.0}; /* Goldish */
+  static const GLfloat mat_specular[]   = { 1.0, 1.0, 1.0, 1.0 };
+  static const GLfloat mat_nospecular[] = { 0.0, 0.0, 0.0, 1.0 };
 
   GLfloat factor;
-  GLfloat line_size = (SITE_MARKER_SIZE/2)/zoom;
+  GLfloat line_size = (SITE_MARKER_SIZE/2.0)/zoom;
 
   GLfloat fromx = tox(fromlat, fromlon);
   GLfloat fromy = toy(fromlat, fromlon);
@@ -222,103 +225,103 @@ plot_sat(const GLfloat fromlat, const GLfloat fromlon,
 #endif
 
   satz = toz(satlat, satlon);
-
+  
   /* Draw the satellite */
-
-    glPushMatrix();
+  
+  glPushMatrix();
+  
+  factor = (GEOSYNC_SAT_ALT+PHYSICAL_EARTH_RADIUS) / PHYSICAL_EARTH_RADIUS;
+  glTranslated(satx*factor,
+	       saty*factor,
+	       satz*factor);
+  
+  glMaterialfv(GL_FRONT, GL_DIFFUSE, satBodyColor);
+  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 80);
+  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
+  
+  gluSphere(sitemarker,SITE_MARKER_SIZE/zoom,7,7);
+  
+  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, satPanelColor);
+  glDisable(GL_CULL_FACE);
+  glBegin(GL_QUADS);
+  {
+    const GLfloat panel_length = 15*SITE_MARKER_SIZE/zoom; 
+    const GLfloat panel_width  = SITE_MARKER_SIZE/zoom; 
     
-    factor = (GEOSYNC_SAT_ALT+PHYSICAL_EARTH_RADIUS) / PHYSICAL_EARTH_RADIUS;
-    glTranslated(satx*factor,
-		 saty*factor,
-		 satz*factor);
-
-    glMaterialfv(GL_FRONT, GL_DIFFUSE, satBodyColor);
-    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 80);
-    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
-
-    gluSphere(sitemarker,SITE_MARKER_SIZE/zoom,7,7);
-
-    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, satPanelColor);
-    glDisable(GL_CULL_FACE);
-    glBegin(GL_QUADS);
-    {
-      const GLfloat panel_length = 15*SITE_MARKER_SIZE/zoom; 
-      const GLfloat panel_width  = SITE_MARKER_SIZE/zoom; 
-      
-      glVertex3f( -panel_width,  -panel_width,  panel_width );
-      glVertex3f( -panel_length, -panel_width,  panel_width );
-      glVertex3f( -panel_length,  panel_width, -panel_width );
-      glVertex3f( -panel_width,   panel_width, -panel_width );
-      
-      glVertex3f(  panel_length, -panel_width,  panel_width );
-      glVertex3f(  panel_width,  -panel_width,  panel_width );
-      glVertex3f(  panel_width,   panel_width, -panel_width );
-      glVertex3f(  panel_length,  panel_width, -panel_width );
-    }
-    glEnd(); // GL_QUADS
-    glEnable(GL_CULL_FACE);
-    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_nospecular);
-
-    glPopMatrix();
-
+    glVertex3f( -panel_width,  -panel_width,  panel_width );
+    glVertex3f( -panel_length, -panel_width,  panel_width );
+    glVertex3f( -panel_length,  panel_width, -panel_width );
+    glVertex3f( -panel_width,   panel_width, -panel_width );
+    
+    glVertex3f(  panel_length, -panel_width,  panel_width );
+    glVertex3f(  panel_width,  -panel_width,  panel_width );
+    glVertex3f(  panel_width,   panel_width, -panel_width );
+    glVertex3f(  panel_length,  panel_width, -panel_width );
+  }
+  glEnd(); // GL_QUADS
+  glEnable(GL_CULL_FACE);
+  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_nospecular);
+  
+  glPopMatrix();
+  
   /* Make the line from -> sat */
-
-    glMaterialfv(GL_FRONT, GL_DIFFUSE, lineColor);
-    glPushMatrix();
-    {
-      GLfloat diffx, diffy, diffz, difflen;
-      
-      glTranslatef(fromx, fromy, fromz);
-      
-      /* rotate around (0,0,1)(satx-fromx, saty-fromy, satz-fromz) */
-      /* angle between a and b is acos(a*b / abs(a)*abs(b)) */
-     
-      diffx   = satx*factor - fromx;
-      diffy   = saty*factor - fromy;
-      diffz   = satz*factor - fromz;
-      difflen = sqrt(diffx*diffx + diffy*diffy + diffz*diffz);
-
-      //      printf("x: %2.3f\ty: %2.3f\tz: %2.3f\tlen: %2.3f\n", diffx, diffy, diffz, difflen);      
-
-      // FIXME Undoubtedly there's an error here, acos only returns angles 
-      //       between 0 and pi, so I better do some thinking.
-      glRotatef(todeg*acos(diffz/difflen),
-		-diffy, diffx, 0.0);
-      
-      gluCylinder(lineobj,
-		  line_size,
-		  line_size,
-		  difflen,
-		  4,
-		  10);    // FIXME tweak this value later.
-    }
-    glPopMatrix();
+  
+  glMaterialfv(GL_FRONT, GL_DIFFUSE, lineColor);
+  glPushMatrix();
+  {
+    GLfloat diffx, diffy, diffz, difflen;
     
+    glTranslatef(fromx, fromy, fromz);
+    
+    /* rotate around (0,0,1)(satx-fromx, saty-fromy, satz-fromz) */
+    /* angle between a and b is acos(a*b / abs(a)*abs(b)) */
+    
+    diffx   = satx*factor - fromx;
+    diffy   = saty*factor - fromy;
+    diffz   = satz*factor - fromz;
+    difflen = sqrt(diffx*diffx + diffy*diffy + diffz*diffz);
+    
+    //      printf("x: %2.3f\ty: %2.3f\tz: %2.3f\tlen: %2.3f\n", diffx, diffy, diffz, difflen);      
+    
+    // FIXME Undoubtedly there's an error here, acos only returns angles 
+    //       between 0 and pi, so I better do some thinking.
+    glRotatef(todeg*acos(diffz/difflen),
+	      -diffy, diffx, 0.0);
+    
+    gluCylinder(lineobj,
+		line_size,
+		line_size,
+		difflen,
+		4,
+		10);    // FIXME tweak this value later.
+  }
+  glPopMatrix();
+  
   /* Make the line dest -> sat */
-
-    glPushMatrix();
-    {
-      GLfloat diffx, diffy, diffz, difflen;
-      
-      glTranslated(destx, desty, destz);
-
-      diffx   = satx*factor - destx;
-      diffy   = saty*factor - desty;
-      diffz   = satz*factor - destz;
-      difflen = sqrt(diffx*diffx + diffy*diffy + diffz*diffz);
-      
-      glRotatef(todeg*acos(diffz/difflen),
-		-diffy, diffx, 0.0);
-      
-      gluCylinder(lineobj,
-		  line_size,
-		  line_size,
-		  difflen,
-		  4,
-		  10);    // FIXME tweak this value later.
-    }      
-    glPopMatrix();
+  
+  glPushMatrix();
+  {
+    GLfloat diffx, diffy, diffz, difflen;
     
+    glTranslated(destx, desty, destz);
+    
+    diffx   = satx*factor - destx;
+    diffy   = saty*factor - desty;
+    diffz   = satz*factor - destz;
+    difflen = sqrt(diffx*diffx + diffy*diffy + diffz*diffz);
+    
+    glRotatef(todeg*acos(diffz/difflen),
+	      -diffy, diffx, 0.0);
+    
+    gluCylinder(lineobj,
+		line_size,
+		line_size,
+		difflen,
+		4,
+		10);    // FIXME tweak this value later.
+  }      
+  glPopMatrix();
+  
 }
 
 /**
@@ -356,7 +359,7 @@ plot_site(const int site_nr, const GLfloat lat, const GLfloat lon,
       printf("Argh! Unknown accuracy!\n");
       break;
     }
-  factor = 1+(2*SITE_MARKER_SIZE*(height))/zoom;
+  factor = 1.0+(2.0*SITE_MARKER_SIZE*(height))/zoom;
   glTranslated(tox(lat, lon)*factor,
 	       toy(lat, lon)*factor,
 	       toz(lat, lon)*factor);
@@ -372,25 +375,20 @@ plot_site(const int site_nr, const GLfloat lat, const GLfloat lon,
   glPopMatrix();
 }
 
-/*-------------------------------------------------------------------------*/
-/* map_texture() : map the current earth_texture to earthobj               */
-/*-------------------------------------------------------------------------*/
+/**
+ * map_texture() : map a texture to earthobj
+ */
+
 void 
-map_texture() 
+map_texture(const GdkPixbuf* tex) 
 {
   GLenum appropriate_format;
-  switch(earth_texture->depth)
-    {
-    case 3:     
-      appropriate_format = GL_RGB;
-      break;
-    case 4: 
-      appropriate_format = GL_RGBA;
-      break;
-    default:
-      g_print("Texture depth not 3 or 4!\n");
-      exit(EXIT_FAILURE);
-    }
+
+  if(gdk_pixbuf_get_has_alpha(tex))
+    appropriate_format = GL_RGBA;
+  else
+    appropriate_format = GL_RGB;
+
 
   gluQuadricTexture(earthobj,GL_TRUE);
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
@@ -400,13 +398,14 @@ map_texture()
      support that. See above for rant on that. */
   gluBuild2DMipmaps(GL_TEXTURE_2D,
 		    GL_RGBA,
-		    earth_texture->width,
-		    earth_texture->height,
+		    gdk_pixbuf_get_width(tex),
+		    gdk_pixbuf_get_height(tex),
 		    appropriate_format,
 		    GL_UNSIGNED_BYTE,
-		    earth_texture->bits);
+		    gdk_pixbuf_get_pixels(tex));
 }
 
+
 /*-------------------------------------------------------------------------*/
 /* set_transparency()                                                      */
 /*-------------------------------------------------------------------------*/
@@ -483,29 +482,30 @@ void set_zoom( gint zoom_type) {
 /*-------------------------------------------------------------------------*/
 void set_view_motion(gfloat strafex, gfloat strafey) 
 {
-       glPopMatrix();
-       glTranslatef((GLfloat) strafex, (GLfloat) strafey,0);
-       glPushMatrix(); /* dummy push so we can pop on model recalc */
+  glPopMatrix();
+  glTranslatef((GLfloat) strafex, (GLfloat) strafey,0);
+  glPushMatrix(); /* dummy push so we can pop on model recalc */
 }
 
 
-/*-------------------------------------------------------------------------*/
-/* init_gl()                                                               */
-/*-------------------------------------------------------------------------*/
-void init_gl(GtkWidget *wi, gpointer data)
+/**
+ * Set up the OpenGL state, init variables etc.
+ */
+
+void 
+init_gl(GtkWidget *wi, gpointer data)
 {
-  int i;
-  GLenum appropriate_format = GL_RGBA;
   
   if (!gtk_gl_area_make_current(GTK_GL_AREA(wi)))
     { 
       printf("make_current failed early in init_gl()\n");
     }
+
   lineobj  = gluNewQuadric();
   earthobj = gluNewQuadric();
   sitemarker = gluNewQuadric();
   gluQuadricDrawStyle(lineobj, GLU_FILL);
-  gluQuadricNormals(lineobj, GLU_SMOOTH);
+//  gluQuadricOrientation(earthobj, GLU_INSIDE); // Wow, that got strange...
   
   
   glEnable(GL_NORMALIZE);  /* Or zoom'll break the lighting */
@@ -542,74 +542,21 @@ void init_gl(GtkWidget *wi, gpointer data)
   glShadeModel(GL_FLAT);
 #endif
   
-  /*  It is BROKEN behavior to support GL_EXT_abgr in gl* but not glu*! 
-      (gluBuild2DMipmaps)
-      
-      Something like
-      
-      #ifdef GL_EXT_abgr
-      if(glutExtensionSupported("GL_EXT_abgr"))
-      {
-      appropriate_format = GL_ABGR_EXT;
-      }
-      else
-      {
-      
-      (byteswap)
-      
-      }
-      #endif
-      would have been so nice here...
-      
-  */
-  
-#if 0
-  
-  // This stuff _might be needed for PCs now. Strange stuff, endianness...
-  
-  if(htons(1234) == 1234)  /* We're on a real machine */
+  switch(user_settings->lighting_mode)
     {
-      /* ABGR -> RGBA */
-#ifdef DEBUG
-      printf("Byteswapping texture...");
-#endif // DEBUG
-      for (i = 0; i < earth_texture->width * earth_texture->height; i++)
-	{
-	  register unsigned char *cp = (unsigned char *) &(earth_texture->bits[i]);
-	  int t;
-	  
-	  t = cp[3];
-	  cp[3] = cp[0];
-	  cp[0] = t;
-	  t = cp[2];
-	  cp[2] = cp[1];
-	  cp[1] = t;
-	}
-#ifdef DEBUG
-      printf("done.");
-#endif //DEBUG
+    case DAY_AND_NIGHT:
+      composite_texture(night_texture, earth_texture, &created_texture);
+      map_texture(created_texture);
+      break;
+    case DAY_ONLY:
+      map_texture(earth_texture);
+      break;
+    default:
+      printf("Funky lighting mode doing textures: %d\n", 
+	     user_settings->lighting_mode);
+      exit(EXIT_FAILURE);
     }
   
-#endif //endif 0
-  
-#if 0
-  gluQuadricTexture(earthobj,GL_TRUE);
-  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-  
-  /* I would be clever and use the nice appropriate_format variable
-     here, but that doesn't work, because gluBuild2DMipmaps doesn't
-     support that. See above for rant on that. */
-  gluBuild2DMipmaps(GL_TEXTURE_2D,
-		    GL_RGB,
-		    earth_texture->width,
-		    earth_texture->height,
-		    appropriate_format,
-		    GL_UNSIGNED_BYTE,
-		    earth_texture->bits);
-#endif //0
-
-  map_texture();       /* s.g */
-
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 #ifdef UGLY_LINES
@@ -618,7 +565,7 @@ void init_gl(GtkWidget *wi, gpointer data)
 #else
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-#endif UGLY_LINES
+#endif //UGLY_LINES
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
   gtk_gl_area_swapbuffers(GTK_GL_AREA(wi));
@@ -669,15 +616,18 @@ void makeearth(void)
   
   glMaterialfv(GL_FRONT, GL_DIFFUSE, defaultColor);
   glEnable(GL_TEXTURE_2D);
-
+  
 #if 0
+  // This doesn't work. For some reason the texture needs to be upside
+  //  down for gluSphere to work. (It generates weird texture
+  //  coordinates)
   gluSphere(earthobj, 1.0, sphere_lod, sphere_lod);
 #else
-  sphere(sphere_lod);
-#endif
-
+  sphere(user_settings->LOD);
+#endif //0
+  
   glDisable(GL_TEXTURE_2D);
-
+  
   glPopMatrix();
   
   for(i=0 ; i<MAX_SITES && sites[i].draw ; i++) /* Here we go */
