@@ -1,3 +1,5 @@
+/* $Id: resolve.c,v 1.4 2003/02/22 23:17:09 d3august Exp $
+*/
 /*  xtraceroute - graphically show traceroute information.
  *  Copyright (C) 1996-1998  Bjrn Augustsson 
  *
@@ -223,13 +225,17 @@ void
 resolve_by_id(int site_nr)
 {
   site* site;
+  static int user_nagged = FALSE;
   
   site = &(sites[site_nr]);
   resolve(site);
   
-  if(site_nr == 0 && site->accuracy < ACC_INCLUDED)
-    tell_user(_("You should enter better\ndata for your site!"));
-
+  if(site_nr == 0 && site->accuracy < ACC_INCLUDED && 
+     user_nagged == FALSE)
+    {
+      tell_user(_("You should enter better\ndata for your site!"));
+      user_nagged = TRUE; 
+    }
   if(site->accuracy == ACC_NONE) /* If it didn't find anything. */
     {
       /* Set the location to the same as the previous site. */
@@ -246,7 +252,7 @@ resolve_by_id(int site_nr)
  */
 
 void 
-arrangeUnknownSites()
+arrangeUnknownSites(void)
 {
   int i;
 
