@@ -1,3 +1,5 @@
+/* $Id: spinner.c,v 1.4 2003/03/02 13:41:42 d3august Exp $
+*/
 /*  xtraceroute - graphically show traceroute information.
  *  Copyright (C) 1996-1998  Bjrn Augustsson 
  *
@@ -55,7 +57,7 @@ gint spinner_spin(gpointer data)
 void spinner_ref(const char* reason)
 {
 
-  dprintf("DEBUG: spinner_ref:\t\"%s\"\tcount: %d -> %d\n",
+  DPRINTF("DEBUG: spinner_ref:\t\"%s\"\tcount: %d -> %d\n",
           reason, refcount, refcount+1);
 
   refcount++;
@@ -72,7 +74,7 @@ void spinner_ref(const char* reason)
 void spinner_unref(const char* reason)
 {
   
-  dprintf("DEBUG: spinner_unref:\t\"%s\"\tcount: %d -> %d\n",
+  DPRINTF("DEBUG: spinner_unref:\t\"%s\"\tcount: %d -> %d\n",
           reason, refcount, refcount-1);
   
   refcount--;
@@ -101,10 +103,11 @@ GtkWidget* spinner_new(void)
   spin  = gtk_button_new();
   label = gtk_label_new(" ");
   gtk_container_add(GTK_CONTAINER(spin), label);
-  
+  /*  
   gtk_signal_connect(GTK_OBJECT(spin), "clicked",
 		     (GtkSignalFunc)new_trace,
 		     (gpointer)NULL);
+  */
   spinner_state = 0;
   gtk_widget_show(label);
 
